//
//  Barcode_MobilePersistenceManager.m
//  Barcode Mobile
//
//  Created by Aaron Wright on 6/27/12.
//  Copyright (c) 2012 A.C. Wright Design. All rights reserved.
//

#import "Barcode_MobilePersistenceManager.h"

static Barcode_MobilePersistenceManager *_defaultManager = nil;

@implementation Barcode_MobilePersistenceManager

@synthesize managedObjectContext = _managedObjectContext;
@synthesize managedObjectModel = _managedObjectModel;
@synthesize persistentStoreCoordinator = _persistentStoreCoordinator;
@synthesize persistentStoreDirectory = _persistentStoreDirectory;

# pragma mark -
# pragma mark init

+ (Barcode_MobilePersistenceManager *)defaultManager {
    @synchronized(self) {
        if (_defaultManager == nil) {
            _defaultManager = [[Barcode_MobilePersistenceManager alloc] init];        
        }
    }
    return _defaultManager;
}

+ (id)allocWithZone:(NSZone *)zone {
    @synchronized(self) {
        if (_defaultManager == nil) {
            return [super allocWithZone:zone];
        }
    }
    return _defaultManager;
}

+ (id)copyWithZone:(NSZone *)zone {
    return self;
}

# pragma mark -
# pragma mark accessors

- (NSUndoManager *)undoManager {
    return [[self managedObjectContext] undoManager];
}

# pragma mark -
# pragma mark actions

- (IBAction)save:(id)sender {
    NSManagedObjectContext *context = [self managedObjectContext];
    [context performBlockAndWait:^{
        if ([context hasChanges]) {
            NSError *error = nil;
            if (![context save:&error]) {
                NSLog(@"Error saving managed object context: %@", error);
            }
        }
    }];
}

# pragma mark -
# pragma mark Core Data

- (NSManagedObjectContext *)managedObjectContext {
    if (_managedObjectContext) {
        return _managedObjectContext;
    }
    
    NSPersistentStoreCoordinator *coordinator = [self persistentStoreCoordinator];
    if (!coordinator) {
        return nil;
    }
    
    _managedObjectContext = [[NSManagedObjectContext alloc] initWithConcurrencyType:NSMainQueueConcurrencyType];
    [_managedObjectContext setMergePolicy:NSMergeByPropertyObjectTrumpMergePolicy];
    [_managedObjectContext setPersistentStoreCoordinator:coordinator];
    
    return _managedObjectContext;
}

- (NSManagedObjectModel *)managedObjectModel
{
    if (_managedObjectModel) {
        return _managedObjectModel;
    }
    
    NSURL *frameworkModelPath = [NSURL fileURLWithPath:[[NSBundle mainBundle] pathForResource:@"Barcode Mobile" ofType:@"momd"]];
	
    _managedObjectModel = [[NSManagedObjectModel alloc] initWithContentsOfURL:frameworkModelPath];
    
    return _managedObjectModel;
}

- (NSURL *)persistentStoreDirectory {
    if (_persistentStoreDirectory) {
        return _persistentStoreDirectory;
    }
    
    _persistentStoreDirectory = [[[NSFileManager defaultManager] URLsForDirectory:NSDocumentDirectory inDomains:NSUserDomainMask] lastObject];    
    
    return _persistentStoreDirectory;
}

- (NSPersistentStoreCoordinator *)persistentStoreCoordinator {
    if (_persistentStoreCoordinator) {
        return _persistentStoreCoordinator;
    }
    
    NSManagedObjectModel *mom = [self managedObjectModel];
    if (!mom) {
        return nil;
    }
    
    _persistentStoreCoordinator = [[NSPersistentStoreCoordinator alloc] initWithManagedObjectModel:mom];
    
    NSURL *url = [[self persistentStoreDirectory] URLByAppendingPathComponent:@"Barcodes.sqlite"];
    NSMutableDictionary *options = [NSMutableDictionary dictionaryWithObjectsAndKeys:
                                    [NSNumber numberWithBool:YES], NSMigratePersistentStoresAutomaticallyOption,
                                    [NSNumber numberWithBool:YES], NSInferMappingModelAutomaticallyOption, 
                                    nil];
    
    dispatch_queue_t queue = dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT,0);
    dispatch_async(queue, ^{
        if ([_persistentStoreCoordinator tryLock]) {
            NSError *error = nil;
            if (![_persistentStoreCoordinator addPersistentStoreWithType:NSSQLiteStoreType configuration:nil URL:[url filePathURL] options:options error:&error]) {
                NSLog(@"Error creating persistent store: %@", error);
            }
            [_persistentStoreCoordinator unlock];
        }
    });
    
    return _persistentStoreCoordinator;
}

@end
